#!/usr/bin/perl -w
#
# GTKSubEditor
#
# gss_callbacks.pl
# Callbacks to interface and some interface parts.
#
# Copyright (c) 2005 Joni Valtanen
# GNU General Public Licence v2

package GSubEditor::callbacks;

use Glib qw(TRUE FALSE);
use strict;
use GSubEditor::config;
use GSubEditor::locale;

use Subtitle;
use 5.008004;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use GSubEditor ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '0.01';


# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

my $video="";

my $importfps=25.0;
my $use_mplayer=1;
my $join_stime=10;
my $lc_language="LANG";
my $writingformat="";

my @subtitles = ();
$subtitles[0]{start}=0;

my $orig_filename="";
my $mainlines=0;
my $filename;
my $workingdir;

sub getFilename
  {
    return $filename;
  }

# updates list_all from $subtitles
# in: list_all->get_model();
sub updateList
  {
    my ($model) = @_;
    my $i=0;

    $model->clear;

    $i=0;
    while ($i < $mainlines) {
      $model->set ($model->append,
		   0, $subtitles[$i]{start},
		   1, $subtitles[$i]{end},
		   2, $subtitles[$i]{starttime},
		   3, $subtitles[$i]{endtime},
		   4, $subtitles[$i]{subtitle},
		   5, TRUE,
		  );
      $i++;
    }


  }

# for quiting program/windows
sub quitProgram
  {
    my ($widget, $window) = @_;
    $window->destroy;
    return 0;
  }

sub delete_event
  {
    my ($window) = @_;
    $window->destroy;
    return TRUE;

  }

sub IsValidTime
  {
    my ($line) = @_;

    my $valid=0;
    my @line_t;
    my $hour=0;
    my $min=0;
    my $sec=0;

    @line_t = split(/:/,$line);

    if ( &Subtitle::IsNumeric( $line_t[0] ) ) {
      $hour = 1;
    }
    if ( &Subtitle::IsNumeric( $line_t[1] ) ) {
      $min = 1;
    }
    if ( &Subtitle::IsDecimal( $line_t[2] ) ) {
      $sec = 1;
    }
    #    print "$sec:$min:$hour\n";

    if ($sec && $min && $hour) {
      $valid = 1;
    }

    return $valid;
  }

sub cell_clicked {
  my ($selection, @cell_data) = @_;
  # assuming you are in single or browse mode -- multisel is a bit different.
  my ($model, $iter) = $selection->get_selected;
  #  my ($cell, $path_string, $column, @cell_data) = @_;
  #  print "list_all\n";

  #  my $model = $cell_data[0][0];
  my $spin_split=$cell_data[0][1];
  my $spin_shift_start=$cell_data[0][2];
  #  my $button_all_apply = $cell_data[0][1];

  #  my $path = Gtk2::TreePath->new_from_string ($path_string);
  #  my $column = $cell->get_data ("column");
  #  my $iter = $model->get_iter ($path);


  #  my $j = ($path->get_indices)[0];
  #  my $line = join("|", ($model->get ($iter))[0])."\n";
  if ($iter) {
    my $line = ($model->get ($iter))[0];
    $spin_split->set_value($line);
    #    $spin_shift_start->set_value($line);
  }
}

# When cell is edited on list_all
sub cell_edited {
  my ($cell, $path_string, $new_text, @cell_data) = @_;

  my $model = $cell_data[0][0];
  my $spin_split=$cell_data[0][1];
  #  my $button_all_apply = $cell_data[0][1];

  my $path = Gtk2::TreePath->new_from_string ($path_string);
  my $column = $cell->get_data ("column");
  my $iter = $model->get_iter ($path);

  #  $button_all_apply->set_sensitive(1);


  if ($column == 0) {
    my $i = ($path->get_indices)[0];
    $subtitles[$i]{start} = $new_text;
    $subtitles[$i]{starttime} = &Subtitle::ConvertFPSToTime($new_text, $importfps);
    ($mainlines, @subtitles) = &Subtitle::NormSubs( $mainlines, $importfps, @subtitles );
    &updateList($model);
    #    $model->set ($iter, $column, $subtitles[$i]{start});

  } elsif ($column == 1) {
    my $i = ($path->get_indices)[0];
    $subtitles[$i]{end} = $new_text;
    $subtitles[$i]{endtime} = &Subtitle::ConvertFPSToTime($new_text, $importfps);
    ($mainlines, @subtitles) = &Subtitle::NormSubs( $mainlines, $importfps, @subtitles );
    &updateList($model);
    #    $model->set ($iter, $column, $subtitles[$i]{end});
  } elsif ($column == 2) {
    my $i = ($path->get_indices)[0];
    my $test = &IsValidTime($new_text);
    if ( $test ) {
      $subtitles[$i]{starttime} = $new_text;
      $subtitles[$i]{start} = &Subtitle::ConvertTimeToFPS($new_text, $importfps);
      ($mainlines, @subtitles) = &Subtitle::NormSubs( $mainlines, $importfps, @subtitles );
      &updateList($model);
    }
    #    $model->set ($iter, $column, $subtitles[$i]{starttime});
  } elsif ($column == 3) {
    my $i = ($path->get_indices)[0];
    my $test = &IsValidTime($new_text);
    if ( $test ) {
      $subtitles[$i]{endtime} = $new_text;
      $subtitles[$i]{end} = &Subtitle::ConvertTimeToFPS($new_text, $importfps);
      ($mainlines, @subtitles) = &Subtitle::NormSubs( $mainlines, $importfps, @subtitles );
      &updateList($model);
    }
    #    $model->set ($iter, $column, $subtitles[$i]{endtime});
  } elsif ($column == 4) {
    my $i = ($path->get_indices)[0];
    $subtitles[$i]{subtitle} = $new_text;
    $model->set ($iter, $column, $subtitles[$i]{subtitle});
  }
}

# When OK button is pressed on open dialog
sub fileOpenOK
  {
    my ($widget, @fook_data) = @_;
    my @rftcl_data;

    my $file_dialog=$fook_data[0][0];
    my $menuitem_edit_join=$fook_data[0][6];
    my $table_split=$fook_data[0][7];

    # gots selected file
    $filename = $file_dialog->get_filename();
    #    $workingdir="/tmp";
    #    $file_dialog->destroy();

    $orig_filename = $filename;

    $fook_data[0][5]->set_text($filename);
    #    $fook_data[0][6]->set_text($importfps);

    $rftcl_data[0]=$filename;
    $rftcl_data[1]=$fook_data[0][2];
    $rftcl_data[2]=$fook_data[0][3];
    $rftcl_data[3]=$fook_data[0][4];

    $importfps = $fook_data[0][16]->get_value();

    #    ($mainlines, @subtitles) = &Subtitle::microdvd::file::ReadFile($filename);
    ($mainlines, @subtitles) = &Subtitle::ReadFile($filename, $importfps);
    &updateList($fook_data[0][2]);

    $menuitem_edit_join->set_sensitive(1);
    $fook_data[0][8]->set_text($importfps);
    $fook_data[0][3]->set_sensitive(1);	#    $button_shift_shift->set_sensitive(1);
    $fook_data[0][4]->set_sensitive(1);	#    $button_convert_convert->set_sensitive(1);
    $fook_data[0][9]->set_sensitive(1);	#    save as
    $fook_data[0][10]->set_sensitive(1); #
    $fook_data[0][11]->set_sensitive(1); #
    $fook_data[0][12]->set_sensitive(1); #

    if($use_mplayer) {
      $fook_data[0][13]->set_sensitive(1); # preview
    } else {
      $fook_data[0][13]->set_sensitive(0); # preview
    }

    $fook_data[0][14]->set_sensitive(0); #    save
    $fook_data[0][15]->set_sensitive(0); #    reread

    if ($mainlines > 1 ) {
      $table_split->set_sensitive(1);
    }

    return 0;
  }

# When OK button is pressed on import dialog
# FIX: to make import functions to other
sub fileImportOK
  {
    my ($widget, @fiok_data) = @_;
    my @rftcl_data;
    my $line;
    my $import_filename="/tmp/gsubeditor-imported.sub";

    my $pwd=$ENV{'PWD'};
    my $file_dialog=$fiok_data[0][0];
    my $menuitem_edit_join=$fiok_data[0][6];
    my $table_split=$fiok_data[0][7];

    #    print "$pwd";

    # gots selected file
    $filename = $file_dialog->get_filename();
    #    $file_dialog->destroy();

    $importfps = $fiok_data[0][16]->get_value();

    chdir "/tmp";
    system("mplayer /dev/zero -rawvideo on:pal:fps=$importfps -vc null -vo null -noframedrop -benchmark -sub \"$filename\" -dumpmicrodvdsub&");
    system("sleep 1 && killall mplayer");
    system("mv ./dumpsub.txt $import_filename");
    chdir "$pwd";
    $filename = $import_filename;
    $orig_filename = $filename;
    $fiok_data[0][5]->set_text($filename); # $label_filename
    #    $fiok_data[0][6]->set_text($importfps); # $label_filename

    $rftcl_data[0]=$filename;
    $rftcl_data[1]=$fiok_data[0][2];
    $rftcl_data[2]=$fiok_data[0][3];
    $rftcl_data[3]=$fiok_data[0][4];

    ($mainlines, @subtitles) = &Subtitle::Microdvd::ReadFile( $filename, $importfps );

    &updateList($fiok_data[0][2]);
    $fiok_data[0][8]->set_text($importfps);

    $menuitem_edit_join->set_sensitive(1);;
    $table_split->set_sensitive(1);
    $fiok_data[0][9]->set_sensitive(1);

    $fiok_data[0][10]->set_sensitive(1);
    $fiok_data[0][11]->set_sensitive(1);
    $fiok_data[0][12]->set_sensitive(1);
    $fiok_data[0][13]->set_sensitive(1);
    $fiok_data[0][14]->set_sensitive(0);
    $fiok_data[0][15]->set_sensitive(0);

    return 0;
  }

# When OK button is pressed on SaveAs dialog
sub fileSaveAsOK
  {
    my ( @fsok_data ) = @_;
    my $oldout;
    my $line;
    my $i;
    my @wf = split(/\*/,$writingformat);
    # gots selected file
    $filename = $fsok_data[0][0]->get_filename; #();
    #    $fsok_data[0][0]->destroy();

    $orig_filename = $filename;

    #write
    &Subtitle::WriteFile($filename, $mainlines, $wf[0] , @subtitles);
    #    print "$filename \n";
    $fsok_data[0][1]->set_sensitive(0);
    $fsok_data[0][2]->set_sensitive(0);
    $fsok_data[0][3]->set_text($filename);
    return 0;
  }

# menu reload calllback
sub fileReread
  {
    my ($widget, @fr_data) = @_;
    my @rftcl_data;

    $filename=$orig_filename;
    #    $rftcl_data[0]=$filename;
    #    $rftcl_data[1]=$fr_data[0][1];
    #    $rftcl_data[2]=$fr_data[0][2];
    #    $rftcl_data[3]=$fr_data[0][3];
    #    $rftcl_data[4]=$fr_data[0][4];

    $fr_data[0][7]->set_text($importfps);

    ($mainlines, @subtitles) = &Subtitle::ReadFile( $filename, $importfps );

    &updateList($fr_data[0][1]);

    $fr_data[0][13]->set_sensitive(0);
    $fr_data[0][14]->set_sensitive(0);

    return 0;
  }

# cancel button on dialogs
sub fileC
  {
    my ($widget, $file_dialog) = @_;

    # gots selected file
    $file_dialog->destroy();

    return 0;
  }

# open dialog
sub fileOpen
  {
    my ($widget, @fo_data) = @_;

    my @fook_data;
    my $file_dialog;
    my $label_open_fps = new Gtk2::Label(&GSubEditor::locale::GetText("OpenFPS"));
    my $open_table = new Gtk2::Table(2,1);
    my $open_adj_fps;
    my $open_spin_fps;

    my @prefs = &GSubEditor::config::getPrefs();

    my $importfps=$prefs[0];
#    $use_mplayer=$prefs[1];
#    $join_stime=$prefs[2];
#    $lc_language=$prefs[3];
#    $writingformat=$prefs[4];

    #    my $filter = Gtk2::FileFilter->new;
    #    $filter->add_pattern("*.xpm"),

    $open_adj_fps = new Gtk2::Adjustment( 0,
					  1.0,
					  100,
					  0.1,
					  1,
					  10 );
    $open_spin_fps = new Gtk2::SpinButton($open_adj_fps, 1.0, 3);
    $open_spin_fps->set_value($importfps);

    $file_dialog = new Gtk2::FileChooserDialog( &GSubEditor::locale::GetText("OpenSubtitleTitle"), undef, 'open', 'gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');
    ##    $file_dialog = new Gtk2::FileSelection( &GSubEditor::locale::GetText("OpenSubtitleTitle") );

    $open_table->set_homogeneous( 0 );
    $open_table->attach( $label_open_fps, 0, 1, 0, 1, "fill", "shrink", 0, 0 );
    $open_table->attach( $open_spin_fps, 1, 2, 0, 1, "fill", "fill", 0, 0 );

    $open_table->show();
    $label_open_fps->show();
    $open_spin_fps->show();

    $open_table->set_border_width( 2 );

    $file_dialog->vbox->add( $open_table  );


    $fook_data[0]=$file_dialog;
    $fook_data[1]=$fo_data[0][0];
    $fook_data[2]=$fo_data[0][1];
    $fook_data[3]=$fo_data[0][2];
    $fook_data[4]=$fo_data[0][3];
    $fook_data[5]=$fo_data[0][4];
    $fook_data[6]=$fo_data[0][5];
    $fook_data[7]=$fo_data[0][6];
    $fook_data[8]=$fo_data[0][7];
    $fook_data[9]=$fo_data[0][8];

    $fook_data[10]=$fo_data[0][9];
    $fook_data[11]=$fo_data[0][10];
    $fook_data[12]=$fo_data[0][11];
    $fook_data[13]=$fo_data[0][12]; # preview
    $fook_data[14]=$fo_data[0][13]; # save
    $fook_data[15]=$fo_data[0][14]; # reread
    $fook_data[16]=$open_spin_fps; # spin

    if ('ok' eq $file_dialog->run) {
      &fileOpenOK( $file_dialog, \@fook_data);
    }

    $file_dialog->destroy();
    ##    $file_dialog->ok_button->signal_connect( "clicked", \&fileOpenOK, \@fook_data );
    ##    $file_dialog->cancel_button->signal_connect( "clicked", \&fileC, $file_dialog );

    ##    $file_dialog->show();

    return 0;
  }

# import dialog
sub fileImport
  {
    my ($widget, @fi_data) = @_;
    my @fiok_data;
    my $file_dialog;
    my $label_import_fps = new Gtk2::Label(&GSubEditor::locale::GetText("OpenFPS"));
    my $import_table = new Gtk2::Table(2,1);
    my $import_adj_fps;
    my $import_spin_fps;

    my @prefs = &GSubEditor::config::getPrefs();

    my $importfps=$prefs[0];
#    $use_mplayer=$prefs[1];
#    $join_stime=$prefs[2];
#    $lc_language=$prefs[3];
#    $writingformat=$prefs[4];

    #    my $filter = Gtk2::FileFilter->new;
    #    $filter->add_pattern("*.xpm"),

    $import_adj_fps = new Gtk2::Adjustment( 0,
					  1.0,
					  100,
					  0.1,
					  1,
					  10 );
    $import_spin_fps = new Gtk2::SpinButton($import_adj_fps, 1.0, 3);
    $import_spin_fps->set_value($importfps);

    $file_dialog = new Gtk2::FileChooserDialog( &GSubEditor::locale::GetText("ImportSubtitleTitle"), undef, 'open', 'gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');

    $import_table->set_homogeneous( 0 );
    $import_table->attach( $label_import_fps, 0, 1, 0, 1, "fill", "shrink", 0, 0 );
    $import_table->attach( $import_spin_fps, 1, 2, 0, 1, "fill", "fill", 0, 0 );

    $import_table->show();
    $label_import_fps->show();
    $import_spin_fps->show();

    $import_table->set_border_width( 2 );

    $file_dialog->vbox->add( $import_table  );

    ##    $file_dialog = new Gtk2::FileSelection( &GSubEditor::locale::GetText("ImportSubtitleTitle"));

    $fiok_data[0]=$file_dialog;
    $fiok_data[1]=$fi_data[0][0];
    $fiok_data[2]=$fi_data[0][1];
    $fiok_data[3]=$fi_data[0][2];
    $fiok_data[4]=$fi_data[0][3];
    $fiok_data[5]=$fi_data[0][4];
    $fiok_data[6]=$fi_data[0][5];
    $fiok_data[7]=$fi_data[0][6];
    $fiok_data[8]=$fi_data[0][7];
    $fiok_data[9]=$fi_data[0][8];

    $fiok_data[10]=$fi_data[0][9];
    $fiok_data[11]=$fi_data[0][10];
    $fiok_data[12]=$fi_data[0][11];
    $fiok_data[13]=$fi_data[0][12];
    $fiok_data[14]=$fi_data[0][13]; # save
    $fiok_data[15]=$fi_data[0][14];
    $fiok_data[16]=$import_spin_fps;

    ##    $file_dialog->ok_button->signal_connect( "clicked", \&fileImportOK, \@fiok_data );

    ##    $file_dialog->cancel_button->signal_connect( "clicked", \&fileC, $file_dialog );
    if ('ok' eq $file_dialog->run) {
      &fileImportOK($file_dialog, \@fiok_data);
    }

    $file_dialog->destroy();
    ##    $file_dialog->show();
    #    print "Import\n";
    return 0;
  }

# save as dialog
sub fileSaveAs
  {
    my ($widget, @fs_data) = @_;
    my @fsok_data;
    my $j=0;
    my $i=0;
    my $save_wr_optionmenu;
    my $save_wr_item;
    my $save_wr_menu;
    my @formats;
    my $default2=0;

    $save_wr_optionmenu = new Gtk2::OptionMenu();
    $save_wr_menu = new Gtk2::Menu();

    $fsok_data[1]=$fs_data[0][13];
    $fsok_data[2]=$fs_data[0][14];
    $fsok_data[3]=$fs_data[0][4];

    my $file_dialog;
    # get Prefs
    my @prefs = &GSubEditor::config::getPrefs();

    $importfps=$prefs[0];
    $use_mplayer=$prefs[1];
    $join_stime=$prefs[2];
    $lc_language=$prefs[3];
    $writingformat=$prefs[4];


    #    my ( @extension ) = split(/\*/, $writingformat );

    $file_dialog = new Gtk2::FileChooserDialog( &GSubEditor::locale::GetText("SaveAsSubtitleTitle"), undef, 'save', 'gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');
    ## FileSelection( &GSubEditor::locale::GetText("SaveAsSubtitleTitle"));

    # build options menu to writing format
    ($j, @formats) = &Subtitle::GetWritableFormats();

    for ($i=0; $i<$j; $i++) {
      #             print "$formats[$i][0].($formats[$i][1])\n";
      $save_wr_item = new Gtk2::MenuItem( $formats[$i][0]." (".$formats[$i][1].")");
      $save_wr_item->signal_connect( "activate", \&activateFormatItem,$formats[$i][0].$formats[$i][1]);
      $save_wr_menu->append( $save_wr_item );
      $save_wr_item->show();
      if ( $writingformat eq $formats[$i][0].$formats[$i][1] ) {
	$default2 = $i;
      }
    }
    $save_wr_optionmenu->set_menu( $save_wr_menu );
    $save_wr_optionmenu->set_history( $default2 );

    my $save_table = new Gtk2::Table(2,1);

    $save_table->set_homogeneous( 0 );
    $save_table->attach( $save_wr_optionmenu, 0, 2, 0, 1, "fill", "shrink", 0, 0 );
#    $save_table->attach_defaults($save_wr_optionmenu, 1, 2 , 0, 1 );

    $file_dialog->vbox->add( $save_table  );
    $save_wr_menu->show();
    $save_wr_optionmenu->show();
    $save_table->show();

    $fsok_data[0]=$file_dialog;


    ##    $file_dialog->complete( "*".$extension[1] );
    ##    $file_dialog->ok_button->signal_connect( "clicked", \&fileSaveAsOK, \@fsok_data );
    ##    $file_dialog->cancel_button->signal_connect( "clicked", \&fileC, $file_dialog );
    ##    $file_dialog->show();
    if ('ok' eq $file_dialog->run) {
      &fileSaveAsOK( \@fsok_data);
    }
    $file_dialog->destroy();
    return 0;
  }

# Closes windows(about/prefs)
sub closeWindow
  {
    my ($widget, $window) = @_;

    $window->destroy();

    return 0;
  }

#shifting callback
sub shifting
  {
    my ($widget, @sdata) = @_;

    my $time =  $sdata[0][0]->value; #$adj_time->value;
    my $start = $sdata[0][1]->value; #$adj_start->value;
    my $end = $sdata[0][2]->value; #$adj_end->value;
    my $line;

    my @rfdata;

    if ( ($filename) && ( $time != 0 ) ) {
      ( $mainlines, @subtitles ) = &Subtitle::Shift( $time, $start, $end, $mainlines, $importfps, @subtitles);

      $rfdata[0]=$filename;	#filename
      $rfdata[1]=$sdata[0][5];	#model
      $rfdata[2]=$sdata[0][6];
      $rfdata[3]=$sdata[0][7];

      updateList($sdata[0][5]);

      $sdata[0][3]->set_sensitive(1); #      $reread_item
      $sdata[0][4]->set_sensitive(1); #      $save_item->set_sensitive(1);

    }

    return 0;
  }


# sets selection $subtitle[x]{start} to spin
sub toStart
  {
    my ($widget, @tsdata) = @_;

    my $text_start;

    my $treeview = $tsdata[0][0];
    my $a_start = $tsdata[0][1];
    my $model = $treeview->get_model;
    my $selection = $treeview->get_selection;
    my $iter = $selection->get_selected;

    if ($iter) {
      my $path = $model->get_path ($iter);
      my $i = ($path->get_indices)[0];
      $text_start=$subtitles[$i]{start};
    }

    $a_start->set_value($text_start); #    $adj_start->set_value($text_start);
    return 0;
  }

# sets selection $subtitle[x]{end} to spin
sub toEnd
  {
    my ($widget, @tedata) = @_;

    my $text_end;

    my $treeview = $tedata[0][0];
    my $a_end = $tedata[0][1];
    my $model = $treeview->get_model;
    my $selection = $treeview->get_selection;
    my $iter = $selection->get_selected;

    if ($iter) {
      my $path = $model->get_path ($iter);
      my $i = ($path->get_indices)[0];
      $text_end=$subtitles[$i]{end};
    }

    $a_end->set_value( $text_end ); #    $adj_end->set_value( $text_end);
    return 0;
  }

# converts subtitle FPS
sub convertFPS
  {
    my ($widget, @cfps_data) = @_;

    my @rfdata;
    my $line;
    my $ofps = $importfps;	#   #$spin_ofps->get_value();
    my $nfps = $cfps_data[0][1]->get_value(); #$spin_nfps->get_value();
    my $cbct = $cfps_data[0][7]->get_active; # gets convert also time

    if ($cbct) {
      ( $mainlines, @subtitles ) = &Subtitle::SetNewOldFPS($ofps,$nfps,$mainlines, @subtitles);
#      print "SetNewOld\n";
    } else {
      ( $mainlines, @subtitles ) = &Subtitle::SetNewFPS($ofps,$nfps,$mainlines, @subtitles);
      $importfps = $nfps;
      $cfps_data[0][0]->set_text( $importfps );
#      print "SetNew\n"
    }

    $rfdata[0]=$filename;	#filename
    $rfdata[1]=$cfps_data[0][4]; #model
    $rfdata[2]=$cfps_data[0][5];
    $rfdata[3]=$cfps_data[0][6];

    &updateList($cfps_data[0][4]);

    $cfps_data[0][2]->set_sensitive(1);	#    $reread_item->set_sensitive(1);
    $cfps_data[0][3]->set_sensitive(1);	#    $save_item->set_sensitive(1);
    return 0;
  }

sub FPS
  {
    my ( $widget, @data ) = @_;

    $data[0][1]->set_value($data[0][0]); #    $spin_ofps->set_value(*spin);
    return 0;

  }

# Edit menuitems

# join dialog OK
sub editJoinOK
  {
    my ($widget, @fjok_data) = @_;

    my $jlines;
    my @jsubs;
    my $start;
    my $end;
    my $subtitle;

    # get last subtitle item from menu
    my $stime;
    my @rftcl_data;

    my $file_dialog=$fjok_data[0][0];

    # gots selected file
    my $join_filename = $file_dialog->get_filename();

    $file_dialog->destroy();
    $stime = $subtitles[ ($mainlines-1) ]{end} + $join_stime; # add ten frames after last frame

    ($jlines, @jsubs) = &Subtitle::ReadFile( $join_filename, $importfps );
    ($jlines, @jsubs) = &Subtitle::Shift( $stime, 0, 0, $jlines, $importfps, @jsubs);
    for my $i (0..$jlines-1) {
      $start=$jsubs[$i]{start};
      $end=$jsubs[$i]{end};
      $subtitle = $jsubs[$i]{subtitle};
      ($mainlines, @subtitles) = &Subtitle::AddLineToEnd($start,$end,$subtitle,$mainlines,$importfps,@subtitles);
    }
    $orig_filename = $filename;

    $rftcl_data[0]=$filename;
    $rftcl_data[1]=$fjok_data[0][2];
    $rftcl_data[2]=$fjok_data[0][3];
    $rftcl_data[3]=$fjok_data[0][4];

    &updateList($fjok_data[0][2]);

    return 0;
  }

# join dialog
sub editJoin
  {
    my ($widget, @fj_data) = @_;

    my @fjok_data;
    my $file_dialog;
    $file_dialog = new Gtk2::FileChooserDialog( &GSubEditor::locale::GetText("ImportSubtitleTitle"), undef, 'open', 'gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');
    ##    $file_dialog = new Gtk2::FileSelection( &GSubEditor::locale::GetText("JoinSubtitleTitle") );
    ##    $file_dialog->complete( "*.sub" );

    $fjok_data[0]=$file_dialog;
    $fjok_data[1]=$fj_data[0][0];
    $fjok_data[2]=$fj_data[0][1];
    $fjok_data[3]=$fj_data[0][2];
    $fjok_data[4]=$fj_data[0][3];
    $fjok_data[5]=$fj_data[0][4];

    ##    $file_dialog->ok_button->signal_connect( "clicked", \&editJoinOK, \@fjok_data );
    ##    $file_dialog->cancel_button->signal_connect( "clicked", \&fileC, $file_dialog );

    ##    $file_dialog->show();
    if ('ok' eq $file_dialog->run) {
      &editJoinOK($file_dialog, \@fjok_data);
    }
    $file_dialog->destroy();

    return 0;
  }

# select language
sub activateLangItem
  {
    my ( $widget, $lang ) = @_;
    $lc_language=$lang;
    return 0;
  }

# select language
sub activateFormatItem
  {
    my ( $widget, $format ) = @_;
    $writingformat=$format;
    return 0;
  }

# sets and save prefs
sub setPrefs
  {
    my ( $widget, @pdata ) = @_;

    my $prefs_window  = $pdata[0][0];
    my $prefs_check_mplayer  = $pdata[0][1];
    my $prefs_spin_import  = $pdata[0][2];
    my $import_item  = $pdata[0][3];
    my $prefs_spin_join  = $pdata[0][4];
    my $current_fps  = $pdata[0][5];
    my $button_preview  = $pdata[0][6];



    $importfps = $prefs_spin_import->get_value();
    $use_mplayer = $prefs_check_mplayer->get_active;
    $join_stime = $prefs_spin_join->get_value();

    $prefs_window->destroy();

    my @prefs=( $importfps, $use_mplayer, $join_stime, $lc_language , $writingformat);
    &GSubEditor::config::setPrefs( @prefs );

    &GSubEditor::config::saveConfigFile();

    if ($use_mplayer) {
      $import_item->set_sensitive(1);
#      $button_preview->set_sensitive(1);
    } else {
      $import_item->set_sensitive(0);
#      $button_preview->set_sensitive(0);
    }


    return 0;
  }

# opens prefs window
sub editPreferences
  {
    my ( $widget, @data ) = @_;

    my $import_item = $data[0][0];
    my $label_current_fps=$data[0][1];
    my $button_preview=$data[0][2];

    my $prefs_window;
    my $prefs_table;
    my $prefs_button_ok;
    my $prefs_button_c;
    my $prefs_text_import_fps;
    my $prefs_text_join_shift;
    my $prefs_text_language;
    my $prefs_text_writing;
    my $prefs_checkbox_mplayer;
    my @prefs_data;
    my $prefs_spin_import_fps;
    my $prefs_adj_import_fps;
    my $prefs_spin_join_shift;
    my $prefs_adj_join_shift;

    my $prefs_lang_optionmenu;
    my $prefs_lang_menu;
    my $prefs_lang_item;

    my $prefs_writing_optionmenu;
    my $prefs_writing_menu;
    my $prefs_writing_item;

    my $k=0;
    my $j=0;
    my $i=0;
    my @formats;
    my @langs;
    my @lang;
    my $line;
    my $default=0;
    my $default2=0;

    # get Prefs
    my @prefs = &GSubEditor::config::getPrefs();

    $importfps=$prefs[0];
    $use_mplayer=$prefs[1];
    $join_stime=$prefs[2];
    $lc_language=$prefs[3];
    $writingformat=$prefs[4];

    # creates about window
    $prefs_window = new Gtk2::Window("toplevel");

    $prefs_table = new Gtk2::Table(2,5);

    $prefs_button_ok = new Gtk2::Button( &GSubEditor::locale::GetText("OK") );
    $prefs_button_c = new Gtk2::Button( &GSubEditor::locale::GetText("Cancel") );
	
    $prefs_checkbox_mplayer = new Gtk2::CheckButton( &GSubEditor::locale::GetText("PrefsCheckboxMPlayer") );
    $prefs_text_import_fps = new Gtk2::Label( &GSubEditor::locale::GetText("PrefsLabelLoadImport") );
    $prefs_text_join_shift = new Gtk2::Label( &GSubEditor::locale::GetText("PrefsLabelJoinShifting") );
    $prefs_text_language = new Gtk2::Label( &GSubEditor::locale::GetText("PrefsLanguage") );

    $prefs_text_writing = new Gtk2::Label( &GSubEditor::locale::GetText("DefaultWritingFormat") );

    $prefs_adj_import_fps = new Gtk2::Adjustment( 0,
						  1.0,
						  100,
						  0.1,
						  1,
						  10 );
    $prefs_adj_join_shift = new Gtk2::Adjustment( 1,
						  0,
						  999999,
						  1,
						  1,
						  10 );

    $prefs_spin_import_fps = new Gtk2::SpinButton($prefs_adj_import_fps, 1.0, 3);
    $prefs_spin_join_shift = new Gtk2::SpinButton($prefs_adj_join_shift, 1, 0);

    $prefs_window->set_title(&GSubEditor::locale::GetText("PrefsTitle") );
    $prefs_window->set_default_size(250,70);

    $prefs_window->signal_connect( "delete_event", sub {$prefs_window->destroy();} );
    $prefs_button_c->signal_connect( "clicked", \&closeWindow, $prefs_window );

    ($j, @langs) = &GSubEditor::locale::GetLanguages();

    $prefs_lang_menu = new Gtk2::Menu();
    $prefs_lang_item = new Gtk2::MenuItem( &GSubEditor::locale::GetText("PrefsLanguageDefault") );
    $prefs_lang_menu->append( $prefs_lang_item );
    $prefs_lang_item->signal_connect( 'activate', \&activateLangItem,"LANG");

    for ($i=0; $i<$j; $i++) {
      #        print "$langs[$i]\n";
      $prefs_lang_item = new Gtk2::MenuItem( $langs[$i] );
      $prefs_lang_menu->append( $prefs_lang_item );
      $prefs_lang_item->signal_connect( "activate", \&activateLangItem,$langs[$i]);
      if ($lc_language) {
	if ($lc_language eq $langs[$i]) {
	  $default= ($i+1);
	}
      }
    }
    $prefs_lang_optionmenu = new Gtk2::OptionMenu();
    $prefs_lang_optionmenu->set_menu($prefs_lang_menu);
    $prefs_lang_optionmenu->set_history($default);

    ($k, @formats) = &Subtitle::GetWritableFormats();
    $prefs_writing_menu = new Gtk2::Menu();
    for ($i=0; $i<$k; $i++) {
      #        print "$formats[$i][0]\n";
      $prefs_writing_item = new Gtk2::MenuItem( $formats[$i][0]." (".$formats[$i][1].")");
      $prefs_writing_menu->append( $prefs_writing_item );
      $prefs_writing_item->signal_connect( "activate", \&activateFormatItem,$formats[$i][0].$formats[$i][1]);
      if ( $writingformat eq $formats[$i][0].$formats[$i][1] ) {
	$default2 = $i;
      }
    }
    $prefs_writing_optionmenu = new Gtk2::OptionMenu();
    $prefs_writing_optionmenu->set_menu($prefs_writing_menu);
    $prefs_writing_optionmenu->set_history($default2);

    $prefs_data[0] = $prefs_window;
    $prefs_data[1] = $prefs_checkbox_mplayer;
    $prefs_data[2] = $prefs_spin_import_fps;
    $prefs_data[3] = $import_item;
    $prefs_data[4] = $prefs_spin_join_shift;
    $prefs_data[5] = $label_current_fps;
    $prefs_data[6] = $button_preview;

    $prefs_button_ok->signal_connect( "clicked", \&setPrefs, \@prefs_data); #$prefs_window);

    $prefs_table->attach_defaults( $prefs_checkbox_mplayer, 1, 2 , 0, 1 );

    $prefs_table->attach_defaults( $prefs_text_import_fps, 0, 1 , 1, 2 );
    $prefs_table->attach_defaults( $prefs_spin_import_fps, 1, 2 , 1, 2 );

    $prefs_table->attach_defaults( $prefs_text_join_shift, 0, 1 , 2, 3 );
    $prefs_table->attach_defaults( $prefs_spin_join_shift, 1, 2 , 2, 3 );

    $prefs_table->attach_defaults( $prefs_text_language, 0, 1 , 3, 4 );
    $prefs_table->attach_defaults( $prefs_lang_optionmenu, 1, 2 , 3, 4 );

    $prefs_table->attach_defaults( $prefs_text_writing, 0, 1 , 4, 5 );
    $prefs_table->attach_defaults( $prefs_writing_optionmenu, 1, 2 , 4, 5 );

    $prefs_table->attach_defaults( $prefs_button_ok, 0, 1 , 5, 6 );
    $prefs_table->attach_defaults( $prefs_button_c, 1, 2 , 5, 6 );

    $prefs_table->set_border_width(4);

    $prefs_button_c->set_border_width(4);
    $prefs_button_ok->set_border_width(4);

    $prefs_spin_import_fps->set_value($importfps);
    $prefs_spin_join_shift->set_value($join_stime);
	
    if ($use_mplayer==1) {
      $prefs_checkbox_mplayer->set_active(1);
    } else {
      $prefs_checkbox_mplayer->set_active(0);
    }
    $prefs_window->set_modal( 1 );
    $prefs_window->add($prefs_table);

    $prefs_window->show_all();
    return 0;
  }


# Splitting

# split dialog begin/end OK
sub splitSaveOK
  {
    my ( $widget, @split_ok_data ) = @_;

    my @split_data;
    my $file_dialog = $split_ok_data[0][0];
    my $from = $split_ok_data[0][1];
    my $to = $split_ok_data[0][2];
    my $shift = $split_ok_data[0][3];
    my $ssb =  $split_ok_data[0][4];

    my $i=0;
    my $j=0;

    my $start=0;
    my $end=0;

    my $line;
    my $oldout;

    $filename = $file_dialog->get_filename();
    $file_dialog->destroy();

    # Save subtitle file from frame to
    # from
    while ( $from > $subtitles[$i]{start} ) {
      $i++;
    }

    # to
    if ( $to==0 ) {
      $shift = $shift - $subtitles[$i]{start};
      $j=$mainlines;
    } else {
      while ( $to > $subtitles[$j]{start} ) {
	$j++;
      }
    }
    $to = $subtitles[$j-1]{start};

    my @wf = split(/\*/,$writingformat);

    &Subtitle::WriteFileFromTo( $filename, $from, $to, $shift, $importfps,  $wf[0], @subtitles );

    if ($ssb) {
      $split_data[0] = $split_ok_data[0][5];
      $split_data[1] = $split_ok_data[0][6];
      splitSaveEnd( $widget, \@split_data );
    }
    return 0;

  }

# split dialog begin
sub splitSave
  {
    my ( $widget, @split_data ) = @_;

    my @split_data2;
    $split_data2[0] = $split_data[0][0];
    $split_data2[1] = $split_data[0][1];

    splitSaveBegin( $widget, \@split_data2 );
    return 0;
  }

# split dialog begin
sub splitSaveBegin
  {
    my ( $widget, @split_data ) = @_;

    my @split_ok_data;
    my $file_dialog;
    my $to = $split_data[0][0]->get_value();
    my @prefs = &GSubEditor::config::getPrefs();
    my $split_wr_optionmenu;
    my $split_wr_item;
    my $split_wr_menu;
    my @formats;
    my $default2=0;
    my $i=0;
    my $j=0;
    $split_wr_optionmenu = new Gtk2::OptionMenu();
    $split_wr_menu = new Gtk2::Menu();


#    my $importfps=$prefs[0];

    $importfps=$prefs[0];
    $use_mplayer=$prefs[1];
    $join_stime=$prefs[2];
    $lc_language=$prefs[3];
    $writingformat=$prefs[4];

#    my ( @extension ) = split(/\*/, $writingformat );



    #    my ( @extension ) = split(/\*/, $writingformat );
    $file_dialog = new Gtk2::FileChooserDialog( &GSubEditor::locale::GetText("SaveSubtitleBeginTitle"), undef, 'save', 'gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');

    ## FileSelection( &GSubEditor::locale::GetText("SaveAsSubtitleTitle"));

    # build options menu to writing format
    ($j, @formats) = &Subtitle::GetWritableFormats();

    for ($i=0; $i<$j; $i++) {
      #             print "$formats[$i][0].($formats[$i][1])\n";
      $split_wr_item = new Gtk2::MenuItem( $formats[$i][0]." (".$formats[$i][1].")");
      $split_wr_item->signal_connect( "activate", \&activateFormatItem,$formats[$i][0].$formats[$i][1]);
      $split_wr_menu->append( $split_wr_item );
      $split_wr_item->show();
      if ( $writingformat eq $formats[$i][0].$formats[$i][1] ) {
	$default2 = $i;
      }
    }
    $split_wr_optionmenu->set_menu( $split_wr_menu );
    $split_wr_optionmenu->set_history( $default2 );

    my $split_table = new Gtk2::Table(2,1);

    $split_table->set_homogeneous( 0 );
    $split_table->attach( $split_wr_optionmenu, 0, 2, 0, 1, "fill", "shrink", 0, 0 );
#    $split_table->attach_defaults($split_wr_optionmenu, 1, 2 , 0, 1 );

    $file_dialog->vbox->add( $split_table  );
    $split_wr_menu->show();
    $split_wr_optionmenu->show();
    $split_table->show();

    ##    $file_dialog = new Gtk2::FileSelection( &GSubEditor::locale::GetText("SaveSubtitleBeginTitle") );
    ##    $file_dialog->complete( "*".$extension[1] );

    $split_ok_data[0]=$file_dialog;
    $split_ok_data[1]=0;	#from
    $split_ok_data[2]=$to;	#to
    $split_ok_data[3]=0;	#to
    $split_ok_data[4]=1;	#to

    $split_ok_data[5] = $split_data[0][0];
    $split_ok_data[6] = $split_data[0][1];

    ##    $file_dialog->ok_button->signal_connect( "clicked", \&splitSaveOK, \@split_ok_data );
    ##    $file_dialog->cancel_button->signal_connect( "clicked", \&fileC, $file_dialog );

    ##    $file_dialog->set_modal( 1 );
    ##    $file_dialog->show();
    if ('ok' eq $file_dialog->run) {
      &splitSaveOK($file_dialog, \@split_ok_data);
    }
    $file_dialog->destroy();
    return 0;
  }

# split dialog end
sub splitSaveEnd
  {
    my ( $widget, @split_data ) = @_;

    my @split_ok_data;
    my $file_dialog;
    my $from = $split_data[0][0]->get_value();
    my $shift = $split_data[0][1]->get_value();
    my @prefs = &GSubEditor::config::getPrefs();
    my $split_wr_optionmenu;
    my $split_wr_item;
    my $split_wr_menu;
    my @formats;
    my $default2=0;
    my $i=0;
    my $j=0;
    $split_wr_optionmenu = new Gtk2::OptionMenu();
    $split_wr_menu = new Gtk2::Menu();

    $importfps=$prefs[0];
    $use_mplayer=$prefs[1];
    $join_stime=$prefs[2];
    $lc_language=$prefs[3];
    $writingformat=$prefs[4];

#    my ( @extension ) = split(/\*/, $writingformat );

    ##    $file_dialog = new Gtk2::FileSelection( &GSubEditor::locale::GetText("SaveSubtitleEndTitle") );
    ##    $file_dialog->complete( "*".$extension[1] );
    $file_dialog = new Gtk2::FileChooserDialog( &GSubEditor::locale::GetText("SaveSubtitleEndTitle"), undef, 'save', 'gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');

    # build options menu to writing format
    ($j, @formats) = &Subtitle::GetWritableFormats();

    for ($i=0; $i<$j; $i++) {
      #             print "$formats[$i][0].($formats[$i][1])\n";
      $split_wr_item = new Gtk2::MenuItem( $formats[$i][0]." (".$formats[$i][1].")");
      $split_wr_item->signal_connect( "activate", \&activateFormatItem,$formats[$i][0].$formats[$i][1]);
      $split_wr_menu->append( $split_wr_item );
      $split_wr_item->show();
      if ( $writingformat eq $formats[$i][0].$formats[$i][1] ) {
	$default2 = $i;
      }
    }
    $split_wr_optionmenu->set_menu( $split_wr_menu );
    $split_wr_optionmenu->set_history( $default2 );

    my $split_table = new Gtk2::Table(2,1);

    $split_table->set_homogeneous( 0 );
    $split_table->attach( $split_wr_optionmenu, 0, 2, 0, 1, "fill", "shrink", 0, 0 );

    $file_dialog->vbox->add( $split_table  );
    $split_wr_menu->show();
    $split_wr_optionmenu->show();
    $split_table->show();

    $split_ok_data[0]=$file_dialog;
    $split_ok_data[1]=$from;	#from
    $split_ok_data[2]=0;	#to
    $split_ok_data[3]=$shift;	#shifting
    $split_ok_data[4]=0;	#to

    ##    $file_dialog->ok_button->signal_connect( "clicked", \&splitSaveOK, \@split_ok_data );
    ##    $file_dialog->cancel_button->signal_connect( "clicked", \&fileC, $file_dialog );

    ##    $file_dialog->set_modal( 1 );
    ##    $file_dialog->show();

    if ('ok' eq $file_dialog->run) {
      &splitSaveOK($file_dialog, \@split_ok_data);
    }
    $file_dialog->destroy();

    return 0;
  }

sub addSubtitle
  {
    my ( $widget, @add_data ) = @_;
    my $start = 1;
    my $end;
    my $subtitle = &GSubEditor::locale::GetText("EditThis");
    my $list_all = $add_data[0][0];

    my $model = $list_all->get_model;
    if ( ($mainlines - 1) >= 0) {
      $start = $subtitles[$mainlines-1]{end} + 1;
    }
    $end = $start + 5;

    ($mainlines, @subtitles) = &Subtitle::AddLineToEnd( $start, $end, $subtitle, $mainlines, $importfps,@subtitles );
    &updateList( $model );
    if ($mainlines > 0 ) {
      $add_data[0][1]->set_sensitive(1);
      $add_data[0][2]->set_sensitive(1);
      $add_data[0][3]->set_sensitive(1);
      $add_data[0][4]->set_sensitive(1);
      $add_data[0][5]->set_sensitive(1);
      $add_data[0][6]->set_sensitive(1);
      $add_data[0][8]->set_sensitive(1);
    }

    #enable split if two or more subs
    if ($mainlines > 1 ) {
      $add_data[0][7]->set_sensitive(1);
    }

    return 0;
  }

sub removeSubtitle
  {
    my ( $widget,  @rm_data) = @_;

    my $list_all = $rm_data[0][0];
    my $model = $list_all->get_model;
    my $selection = $list_all->get_selection;
    my $iter = $selection->get_selected;
    if ($iter) {
      my $path = $model->get_path ($iter);
      my $i = ($path->get_indices)[0];
      delete $subtitles[$i];
    }

    ($mainlines, @subtitles) = &Subtitle::NormSubs( $mainlines, $importfps, @subtitles );
    &updateList( $model );
    if ($mainlines <= 0) {
      $widget->set_sensitive(0);
      $rm_data[0][2]->set_sensitive(0);
      $rm_data[0][3]->set_sensitive(0);
      $rm_data[0][4]->set_sensitive(0);
      $rm_data[0][5]->set_sensitive(0);
      $rm_data[0][6]->set_sensitive(0);
      $rm_data[0][8]->set_sensitive(0);
    }
    if ($mainlines <= 1) {
      $rm_data[0][7]->set_sensitive(0);
    }
    return 0;
  }

sub preview
  {
    my ( $widget, $list_all ) = @_;
    my $model = $list_all->get_model;
    my $selection = $list_all->get_selection;
    my $iter = $selection->get_selected;
    my $tmpsub="";
    my $line;
    #    my $video="/var/storage/movie/das.kondom.des.grauens.avi";

    my @starttime;
    #    $starttime[0]=-1;

    if ($video eq "") {
      previewVideo();
    }
    if (int($subtitles[0]{start})>0) {
      $tmpsub="/tmp/gsubeditor-preview.sub";
      &Subtitle::WriteFile( $tmpsub, $mainlines, "MICRODVD", @subtitles);
      #      system("rm $tmpsub");
    }

    if ($video) {

      if ($iter) {
	my $path = $model->get_path ($iter);
	my $i = ($path->get_indices)[0];
	@starttime = split(/\./,$subtitles[$i]{starttime});
      }

      if (!$starttime[0]) {
	$starttime[0]=0;
      }
      if (int($subtitles[0]{start})>0) {
	$line = "mplayer -sub $tmpsub -ss $starttime[0] $video&";
      } else {
	$line = "mplayer -ss $starttime[0] $video&";
      }

      system( "$line" );
    }
    return 0;
  }

sub previewVideoOK
  {
    my ( $widget, @ok_data ) = @_;

    my $file_dialog = $ok_data[0][0];

    $video = $file_dialog->get_filename();
    $video = "\"$video\"";
    $file_dialog->destroy();

    return 0;

  }

# split dialog begin
sub previewVideo
  {
    my ( $widget ) = @_;

    my $file_dialog;
    my @ok_data;

    #    my @prefs = &GSubEditor::config::getPrefs();

    #    $importfps=$prefs[0];
    #    $use_mplayer=$prefs[1];
    #    $join_stime=$prefs[2];
    #    $lc_language=$prefs[3];
    #    $writingformat=$prefs[4];

    $file_dialog = new Gtk2::FileChooserDialog( &GSubEditor::locale::GetText("PreviewVideoTitle"), undef, 'open', 'gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');
    ##    $file_dialog = new Gtk2::FileSelection( &GSubEditor::locale::GetText("PreviewVideoTitle") );
    ##    $file_dialog->complete( "*" );

    $ok_data[0]=$file_dialog;

    ##    $file_dialog->ok_button->signal_connect( "clicked", \&previewVideoOK, \@ok_data );
    ##    $file_dialog->cancel_button->signal_connect( "clicked", \&fileC, $file_dialog );

    ##    $file_dialog->set_modal( 1 );
    ##    $file_dialog->show();
    if ('ok' eq $file_dialog->run) {
      &previewVideoOK($file_dialog, \@ok_data);
    }

    $file_dialog->destroy();
    return 0;
  }

sub fileSave
  {
    my ($widget, @fs_data) = @_;

    my @wf = split(/\./,$orig_filename);
    my @prefs = &GSubEditor::config::getPrefs();

    $wf[0] = pop(@wf);

    $wf[0] = &Subtitle::GetFiletype( $wf[0] );

    if ($orig_filename && $wf[0]) {
      #write
      &Subtitle::WriteFile($orig_filename, $mainlines, $wf[0] , @subtitles);
      $widget->set_sensitive(0);
      $fs_data[0][13]->set_sensitive(0);
      $fs_data[0][14]->set_sensitive(0);
    } else {
      fileSaveAS();
    }
  }
1;
__END__

=head1 NAME

GSubEditor - Perl extension only for gsubeditor script

=head1 SYNOPSIS

  use GSubEditor;

=head1 DESCRIPTION

  Is used by gsubeditor to manage subtitles with Gtk2-interface.

=head2 EXPORT

None by default.

=head1 SEE ALSO

Subtitle(pm3), GSubEditor(pm3), GSubEditor::config(pm3), GSubEditor::locale(pm3), GSubEditor::interface(pm3)

http://www.kapsi.fi/~jvaltane/subtitle/
http://www.kapsi.fi/~jvaltane/gsubeditor/

=head1 AUTHOR

Joni Valtanen, E<lt>jvaltane@kapsi.fiE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006 by Joni Valtanen

This library is free software; you can redistribute it and/or modify
it under the GNU General Public Licence version 2.

Read COPYING from the package for further information.


=cut
