#!/usr/bin/perl -w
#
# GTKSubEditor
#
# gss_interface.pl
# Creates interface
#
# Copyright (c) 2005 Joni Valtanen
# GNU General Public Licence v2
#
package GSubEditor::interface;

use strict;
use Glib qw(TRUE FALSE);

use GSubEditor::callbacks;
use GSubEditor::locale;
use 5.008004;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use GSubEditor ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '0.01';


# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

sub createAbout
  {

    # to the about window
    my $about_window;
    my $about_button;
    my $about_text;
    my $about_table;

    # creates about window
    $about_window = new Gtk2::Window("toplevel");
    $about_table = new Gtk2::Table(2,2);
    $about_button = new Gtk2::Button( &GSubEditor::locale::GetText("OK") );
    $about_text = new Gtk2::Label( &GSubEditor::locale::GetText("Copyright") );

    $about_window->set_title( &GSubEditor::locale::GetText("AboutWindowTitle") );
    $about_window->set_default_size(250,70);
    $about_window->signal_connect( "delete_event", sub { $about_window->destroy();} );
    $about_button->signal_connect( "clicked", \&GSubEditor::callbacks::closeWindow, $about_window );

    $about_table->attach_defaults( $about_text, 0, 1 , 0, 1 );
    $about_table->attach_defaults( $about_button, 0, 1 , 1, 2 );
    $about_table->set_border_width( 4 );

    $about_button->set_border_width( 4 );

    $about_window->set_modal( 1 );
    $about_window->add( $about_table );

    $about_window->show_all();
    return 0;
  }

# Creates List (list_all) to scrolled window
sub createList
  {
    my (@spins) = @_;

    my $renderer;
    my $model_all;
    my $list_all;
    my @cell_data;

    $model_all = Gtk2::ListStore->new ('Glib::Uint', 'Glib::Uint',
				       'Glib::String', 'Glib::String',
				       'Glib::String', 'Glib::Boolean');

    $list_all = Gtk2::TreeView->new ($model_all);
    $list_all->set_rules_hint (TRUE);

    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 0);

    $cell_data[0]=$model_all;
    $cell_data[1]=$spins[0][0]; # split
    $cell_data[2]=$spins[0][1]; # shift

    $renderer->signal_connect (edited => \&GSubEditor::callbacks::cell_edited, \@cell_data);
    $list_all->get_selection->signal_connect (changed => \&GSubEditor::callbacks::cell_clicked, \@cell_data);

    #    $list_all->signal_connect ('row-activated' => \&GSubEditor::callbacks::cell_clicked, \@cell_data);

    $list_all->insert_column_with_attributes (-1,  &GSubEditor::locale::GetText("StartFrame"),
					      $renderer,
					      text => 0,
					      editable => 5
					     );

    # column for severities
    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 1);
    $renderer->signal_connect (edited => \&GSubEditor::callbacks::cell_edited, \@cell_data);
    $list_all->insert_column_with_attributes (-1, &GSubEditor::locale::GetText("StopFrame"),
					      $renderer,
					      text => 1,
					      editable => 5
					     );
    # column for severities
    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 2);
    $renderer->signal_connect (edited => \&GSubEditor::callbacks::cell_edited, \@cell_data);
    $list_all->insert_column_with_attributes (-1,  &GSubEditor::locale::GetText("StartTime"),
					      $renderer,
					      text => 2,
					      editable => 5
					     );
    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 3);
    $renderer->signal_connect (edited => \&GSubEditor::callbacks::cell_edited, \@cell_data);
    $list_all->insert_column_with_attributes (-1,  &GSubEditor::locale::GetText("StopTime"),
					      $renderer,
					      text => 3,
					      editable => 5
					     );
    # column for description
    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 4);
    $renderer->signal_connect (edited => \&GSubEditor::callbacks::cell_edited, \@cell_data);
    $list_all->insert_column_with_attributes (-1, &GSubEditor::locale::GetText("Subtitle"),
					      $renderer,
					      text => 4,
					      editable => 5
					     );

    return( $list_all, $model_all, $renderer );
  }


#Creates main window
sub createGUI
  {

    # main window
    my $window;

    # buttons Shift
    my $button_shift_shift;
    my $button_shift_start;
    my $button_shift_end;

    # buttons Convert
    my $button_convert_convert;

    #spins Convert
    my $adj_convert_ofps;
    my $adj_convert_nfps;
    my $spin_convert_ofps;
    my $spin_convert_nfps;

    my $checkbox_convert_time;

    my $label_all_filename;

    my $button_all_remove;
    my $button_all_add;

    my $button_all_preview;

    #adjusters
    my $adj_shift_time;
    my $adj_shift_start;
    my $adj_shift_end;

    #    $list_all = 0;

    my $notebook;
    my $frame_all_subfile;

    my $table_main;
    my $table_filename;
    my $table_split;
    my $table_join;
    my $table_convert;
    my $table_shift;
    my $table_all;

    my $button_quit;

    #optionmenu Convert
    #    my $option_ofps;
    my $option_nfps;
    #    my $option_ofps_menu;
    my $option_nfps_menu;

    #    my $ofps29_item;
    #    my $ofps25_item;
    #    my $ofps23_item;

    my $nfps29_item;
    my $nfps25_item;
    my $nfps23_item;


    #menus
    my $menu_bar;
    my $menuitem_file;
    my $menuitem_edit;
    my $menuitem_help;

    my $menu_file;
    my $menuitem_separator;
    my $menuitem_separator2;
    my $menuitem_file_import;
    my $menuitem_file_reread;
    my $menuitem_file_save;
    my $menuitem_file_saveas;
    my $menuitem_file_open;
    my $menuitem_file_previewvideo;
    my $menuitem_file_quit;

    my $menu_edit;
    my $menuitem_edit_join;
    my $menuitem_edit_prefs;

    my $menu_help;
    my $menuitem_help_about;

    #labels
    my $label_notebook_shift;
    my $label_notebook_convert;
    my $label_notebook_tconvert;
    my $label_notebook_split;

    my $label_shift_start;
    my $label_shift_end;
    my $label_shift_time;

    my $label_convert_ofps;
    my $label_convert_cfps;
    my $label_convert_nfps;

    # Shift
    my $spin_shift_time;
    my $spin_shift_start;
    my $spin_shift_end;

    my $label_shift_sframes;
    my $label_shift_eframes;
    my $label_shift_tframes;

    # Split
    my $label_split_second;
    my $spin_split_second;
    my $adj_split_second;

    my $label_split_secondstarts;
    my $spin_split_secondstarts;
    my $adj_split_secondstarts;

    #    my $button_split_second;
    my $button_split_save;
    #    my $button_split_end;


    #Scolled window & list
    my $scrolled_window_all;
    my $model_all;
    my $list_all;
    my $renderer_all;

    #other
    my $false = 0;
    my $true = 1;

    #datas for callbacks
    my @s_data;			# shifting data
    my @ts_data;		# toStart data
    my @te_data;		# toEnd data
    my @cfps_data;		# convertFPS data
    my @add_data;
    my @ss_data;		# split to start data
    my @split_data;		# split data

    my @f_data;			#file dialogs data
    my @p_data;			#external prefs data

    my @ofps25_data;
    my @ofps23_data;
    my @ofps29_data;

    my @nfps25_data;
    my @nfps23_data;
    my @nfps29_data;

    my @spins;

    my @prefs;

    (@prefs) = &GSubEditor::config::getPrefs();
    my $importfps=$prefs[0];
    my $use_mplayer=$prefs[1];
    my $join_stime=$prefs[2];
    my $lc_language=$prefs[3];
    my $writingformat=$prefs[4];

    &GSubEditor::locale::SetLanguage($lc_language);
    &GSubEditor::locale::LoadLanguages();

    $window = new Gtk2::Window( "toplevel" );
    $window->set_default_size(500,400);


    # basic stings
    $frame_all_subfile = new Gtk2::Frame( &GSubEditor::locale::GetText("Subtitle File") );
    $frame_all_subfile->set_border_width(4);

    $table_main = new Gtk2::Table( 4, 2, $true );
    $table_filename = new Gtk2::Table( 1, 1, $true );
    $table_shift = new Gtk2::Table( 5, 4, $true );
    $table_convert = new Gtk2::Table( 3, 3, $true );
    $table_all = new Gtk2::Table( 2, 4, $true );

    $table_split = new Gtk2::Table( 5, 4, $true );
    $table_join = new Gtk2::Table( 5, 4, $true );

	
    #notebook
    $notebook = new Gtk2::Notebook();

    # Shift buttons
    $button_shift_shift = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonShift") );
    $button_shift_start = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonFromSelection") );
    $button_shift_end = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonFromSelection" ) );
    $button_quit = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonQuit") );

    # Split buttons
    #    $button_split_second = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonFromSelection") );
    $button_split_save = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonSaveSplitted") );
    #    $button_split_end = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonSaveEnd") );

    # Convert buttons
    $button_convert_convert = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonConvertFPS") );

    # Convert option menus
    #    $option_ofps = new Gtk2::OptionMenu();
    $option_nfps = new Gtk2::OptionMenu();
	
    # menus
    $menu_bar = new Gtk2::MenuBar();
    $menuitem_file = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuFile") );
    $menuitem_edit = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuEdit") );
    $menuitem_help = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuHelp") );

    $menuitem_separator = new Gtk2::MenuItem( );
    $menuitem_separator2 = new Gtk2::MenuItem( );

    $menu_file = new Gtk2::Menu();
    $menuitem_file_open = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuOpen" ) );
    $menuitem_file_import = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuImport") );
    $menuitem_file_reread = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuReload" ) );
    $menuitem_file_saveas = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuSaveAs" ) );
    $menuitem_file_save = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuSave" ) );
    $menuitem_file_previewvideo = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuPreviewVideo" ) );
    $menuitem_file_quit = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuQuit") );

    $menu_edit = new Gtk2::Menu();
    $menuitem_edit_join = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuJoinAfter") );
    $menuitem_edit_prefs = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuPrefs") );

    $menu_help = new Gtk2::Menu();
    $menuitem_help_about = new Gtk2::MenuItem( &GSubEditor::locale::GetText("MenuAbout") );

    # option menus Convert
    #    $option_ofps_menu = new Gtk2::Menu();
    $option_nfps_menu = new Gtk2::Menu();

    #    $ofps23_item = new Gtk2::MenuItem( "23.976" );
    #    $ofps25_item = new Gtk2::MenuItem( "25" );
    #    $ofps29_item = new Gtk2::MenuItem( "29" );

    $nfps23_item = new Gtk2::MenuItem( "23.976" );
    $nfps25_item = new Gtk2::MenuItem( "25" );
    $nfps29_item = new Gtk2::MenuItem( "29" );

    #    $option_ofps_menu->append( $ofps23_item );
    #    $option_ofps_menu->append( $ofps25_item );
    #    $option_ofps_menu->append( $ofps29_item );

    $option_nfps_menu->append( $nfps23_item );
    $option_nfps_menu->append( $nfps25_item );
    $option_nfps_menu->append( $nfps29_item );


    # labels
    $label_notebook_shift = new Gtk2::Label( &GSubEditor::locale::GetText("LabelShift") );
    $label_notebook_convert = new Gtk2::Label( &GSubEditor::locale::GetText("LabelConvert") );
    $label_notebook_split = new Gtk2::Label( &GSubEditor::locale::GetText("LabelSplit") );

    $label_shift_start = new Gtk2::Label( &GSubEditor::locale::GetText("LabelStart:"));
    $label_shift_end = new Gtk2::Label( &GSubEditor::locale::GetText("LabelEnd:") );
    $label_shift_time = new Gtk2::Label( &GSubEditor::locale::GetText("LabelTime:") );

    $label_shift_sframes = new Gtk2::Label( &GSubEditor::locale::GetText("LabelFromFrame") );
    $label_shift_eframes = new Gtk2::Label( &GSubEditor::locale::GetText("LabelToFrame") );
    $label_shift_tframes = new Gtk2::Label( &GSubEditor::locale::GetText("LabelFrames"));

    $label_shift_start->set_justify( "left" );
    $label_shift_end->set_justify( "left" );
    $label_shift_time->set_justify( "left" );

    #Other labels
    $label_split_second = new Gtk2::Label( &GSubEditor::locale::GetText("LabelSplitSecondFrom"));
    $label_split_secondstarts = new Gtk2::Label( &GSubEditor::locale::GetText("LabelSecondStartsAt") );

    $label_all_filename = new Gtk2::Label( &GSubEditor::locale::GetText("LabelNotLoaded"));

    $label_convert_ofps = new Gtk2::Label( &GSubEditor::locale::GetText("LabelCurrentFPS"));
    $label_convert_cfps = new Gtk2::Label( $importfps );
    $label_convert_nfps = new Gtk2::Label( &GSubEditor::locale::GetText("LabelNewFPS"));
	
    #adjustments to spin bittons
    $adj_shift_time = new Gtk2::Adjustment( 0,
					    -999999,
					    999999,
					    1,
					    1,
					    10 );

    $adj_shift_start = new Gtk2::Adjustment( 0,
					     0,
					     999999,
					     1,
					     1,
					     10 );

    $adj_shift_end = new Gtk2::Adjustment( 0,
					   0,
					   999999,
					   1,
					   1,
					   10 );

    #Convert adjustments
    #    $adj_convert_ofps = new Gtk2::Adjustment( 0,
    #					    1.0,
    #					    100,
    #					    0.1,
    #					    1,
    #					    10 );

    $adj_convert_nfps = new Gtk2::Adjustment( 0,
					      1.0,
					      100,
					      0.1,
					      1,
					      10 );

    #Split adjustements
    $adj_split_second = new Gtk2::Adjustment( 0,
					      0,
					      999999,
					      1,
					      1,
					      10 );
    $adj_split_secondstarts = new Gtk2::Adjustment( 0,
						    1,
						    999999,
						    1,
						    1,
						    10 );

    # Shift spinbuttons
    $spin_shift_start = new Gtk2::SpinButton($adj_shift_start, 1.0, 0);
    $spin_shift_end = new Gtk2::SpinButton($adj_shift_end, 1.0, 0);
    $spin_shift_time = new Gtk2::SpinButton($adj_shift_time, 1.0, 0);

    # Convert spins
    #    $spin_convert_ofps = new Gtk2::SpinButton($adj_convert_ofps, 1.0, 3);
    $spin_convert_nfps = new Gtk2::SpinButton($adj_convert_nfps, 1.0, 3);
    #    $spin_convert_ofps->set_value( "23.976" );
    $spin_convert_nfps->set_value( "23.976" );

    $checkbox_convert_time = new Gtk2::CheckButton( &GSubEditor::locale::GetText("CheckboxConvertTime") );

    # Split spins
    $spin_split_second = new Gtk2::SpinButton($adj_split_second, 1.0, 0);
    $spin_split_secondstarts = new Gtk2::SpinButton($adj_split_secondstarts, 1.0, 0);
    $spin_split_secondstarts->set_value( "1" );

    $button_all_add = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonAdd") );
    $button_all_remove = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonRemove") );
    $button_all_preview = new Gtk2::Button( &GSubEditor::locale::GetText("ButtonPreview") );

    # list	
    $scrolled_window_all = new Gtk2::ScrolledWindow( undef, undef );
    $scrolled_window_all->set_policy( 'automatic', 'always' );
    $scrolled_window_all->show();

    @spins = ($spin_split_second, $spin_shift_start);
    #Create list
    ($list_all, $model_all, $renderer_all) = &createList(\@spins);

    # File menu
    $menu_file->append( $menuitem_file_open );
    $menu_file->append( $menuitem_file_save );
    $menu_file->append( $menuitem_file_saveas );
    $menu_file->append( $menuitem_file_reread );

    $menu_file->append( $menuitem_separator );
    $menu_file->append( $menuitem_file_previewvideo );
    $menu_file->append( $menuitem_file_import );
    $menu_file->append( $menuitem_separator2 );

    $menu_file->append( $menuitem_file_quit );

    #    $menuitem_file->show();
    $menuitem_file->set_submenu( $menu_file );

    $menu_file->show();

    #show option menu
    #    $option_ofps->set_menu($option_ofps_menu);
    $option_nfps->set_menu($option_nfps_menu);

    #    $option_ofps->set_history( 0 );
    $option_nfps->set_history( 0 );

    #Edit menu
    $menu_edit->append( $menuitem_edit_join );
    $menu_edit->append( $menuitem_edit_prefs );
    $menuitem_edit->set_submenu( $menu_edit );

    # Help menu
    $menu_help->append( $menuitem_help_about );
    $menuitem_help->set_submenu( $menu_help );


    # build Convert option menu

    # build menu bar
    $menuitem_file_reread->set_sensitive(0);
    $menuitem_file_saveas->set_sensitive(0);
    $menuitem_file_save->set_sensitive(0);
	
    #notebook
    $notebook->prepend_page( $table_split, $label_notebook_split);
    $notebook->prepend_page( $table_convert, $label_notebook_convert);
    $notebook->prepend_page( $table_shift, $label_notebook_shift );

    # table
    my $xoptions = "shrink";
    my $yoptions = "shrink";
    my $xpadding = 0;
    my $ypadding = 0;

    $scrolled_window_all->add( $list_all );

    # table shift
    $table_shift->set_homogeneous( $false);

    $table_shift->attach( $label_shift_start, 0, 1, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $spin_shift_start, 1, 2, 0, 1, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $label_shift_sframes, 2, 3, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $button_shift_start, 3, 4, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_shift->attach( $label_shift_end, 0, 1, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $spin_shift_end, 1, 2, 1, 2, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $label_shift_eframes, 2, 3, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $button_shift_end, 3, 4, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_shift->attach( $label_shift_time, 0, 1, 2, 3, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $spin_shift_time, 1, 2, 2, 3, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $label_shift_tframes, 2, 3, 2, 3, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_shift->attach( $button_shift_shift, 0, 4, 3, 4, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->set_border_width(4);
    #	$frame_shift->add($table_shift);

    # table convert
    $table_convert->set_homogeneous( $false);
    $table_convert->attach( $label_convert_ofps, 0, 1, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $label_convert_cfps, 1, 2, 0, 1, "fill", $yoptions, $xpadding, $ypadding );
    #    $table_convert->attach( $spin_convert_ofps, 1, 2, 0, 1, ["expand","fill"], $yoptions, $xpadding, $ypadding );
    #    $table_convert->attach( $option_ofps, 2,3,0,1,["expand","fill"], $yoptions, $xpadding, $ypadding );


    $table_convert->attach( $label_convert_nfps, 0, 1, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $spin_convert_nfps, 1, 2, 1, 2,  "fill", $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $option_nfps, 2,3,1,2,["expand","fill"], $yoptions, $xpadding, $ypadding );

    $table_convert->attach( $button_convert_convert, 0, 3, 2, 3, "fill", $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $checkbox_convert_time, 0, 3, 3, 4, "fill", $yoptions, $xpadding, $ypadding );

    $table_convert->set_border_width(4);

    # table split
    $table_split->set_homogeneous( $false);
    $table_split->attach( $label_split_second, 0, 1, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_split->attach( $spin_split_second, 1, 2, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    #    $table_split->attach( $button_split_second, 2, 3, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_split->attach( $label_split_secondstarts, 0, 1, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_split->attach( $spin_split_secondstarts, 1, 2, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_split->attach( $button_split_save, 1, 3, 2, 3, ["expand","fill"], $yoptions, $xpadding, $ypadding );


    $table_split->set_border_width(4);

    # table all
    $table_all->set_homogeneous( $false );

    $frame_all_subfile->add( $label_all_filename ); #, 1, 2, 2, 3, "expand", $yoptions, $xpadding, $ypadding);
    $table_all->attach( $frame_all_subfile, 0, 2, 0, 1, "fill", $yoptions, $xpadding, $ypadding);
    $table_all->attach_defaults( $scrolled_window_all, 0, 2, 1, 3 );

    $table_all->attach( $button_all_add, 0, 1, 3, 4, "fill", $yoptions, $xpadding, $ypadding );
    $table_all->attach( $button_all_remove, 1, 2, 3, 4, "fill", $yoptions, $xpadding, $ypadding );

    #    $table_all->attach( $button_all_add, 0, 1, 3, 4, "fill", $yoptions, $xpadding, $ypadding );
    $table_all->attach( $button_all_preview, 0, 2, 4, 5, "fill", $yoptions, $xpadding, $ypadding );

    $table_all->attach( $button_quit, 1, 2, 5, 6, "fill", $yoptions, $xpadding, $ypadding );
    $table_all->set_border_width(4);

    # table main
    $table_main->set_homogeneous( $false );

    $table_main->attach( $menu_bar, 0, 2, 0, 1, "fill", $yoptions, $xpadding, $ypadding);
    $table_main->attach( $notebook, 0, 2, 1, 2, "fill", $yoptions, $xpadding, $ypadding);
    $table_main->attach_defaults( $table_all, 0, 2, 3, 4);
    #	$table_main->set_border_width(2);

    $menu_bar->append($menuitem_file);	
    $menu_bar->append($menuitem_edit);	
    $menu_bar->append($menuitem_help);	

    #    $button_shift_shift->set_sensitive(0);
    #    $button_convert_convert->set_sensitive(0);
    #    $button_all_remove->set_sensitive(1);

    $menuitem_edit_join->set_sensitive(0);

    if ($use_mplayer) {
      $menuitem_file_import->set_sensitive(1);
      $button_all_preview->set_sensitive(1);
    } else {
      $menuitem_file_import->set_sensitive(0);
      $button_all_preview->set_sensitive(0);
    }

    $f_data[0]=&GSubEditor::callbacks::getFilename(); #$main::filename;
    $f_data[1]=$model_all;
    $f_data[2]=$button_shift_shift;
    $f_data[3]=$button_convert_convert;
    $f_data[4]=$label_all_filename;
    $f_data[5]=$menuitem_edit_join;
    $f_data[6]=$table_split;
    $f_data[7]=$label_convert_cfps;
    $f_data[8]=$menuitem_file_saveas;

    $f_data[9]=$table_shift;
    $f_data[10]=$table_convert;
    $f_data[11]=$button_all_remove;
    $f_data[12]=$button_all_preview;
    $f_data[13]=$menuitem_file_save;
    $f_data[14]=$menuitem_file_reread;

    # signal for menus
    $menuitem_file_open->signal_connect( 'activate', \&GSubEditor::callbacks::fileOpen, \@f_data );
    $menuitem_file_import->signal_connect( 'activate', \&GSubEditor::callbacks::fileImport, \@f_data );
    $menuitem_file_reread->signal_connect( 'activate', \&GSubEditor::callbacks::fileReread, \@f_data );
    $menuitem_file_saveas->signal_connect( 'activate', \&GSubEditor::callbacks::fileSaveAs, \@f_data );
    $menuitem_file_save->signal_connect( 'activate', \&GSubEditor::callbacks::fileSave, \@f_data );
    $menuitem_file_previewvideo->signal_connect( 'activate', \&GSubEditor::callbacks::previewVideo );
    $menuitem_file_quit->signal_connect( 'activate', \&GSubEditor::callbacks::quitProgram, $window );

    $menuitem_edit_join->signal_connect( 'activate', \&GSubEditor::callbacks::editJoin, \@f_data );

    $p_data[0]=$menuitem_file_import;
    $p_data[1]=$label_convert_cfps;
    $p_data[2]=$button_all_preview;

    $menuitem_edit_prefs->signal_connect( 'activate', \&GSubEditor::callbacks::editPreferences, \@p_data );

    $menuitem_help_about->signal_connect( 'activate', \&createAbout );

    #    $ofps25_data[0]="25.0";
    #    $ofps25_data[1]=$spin_convert_ofps;
    #    $ofps23_data[0]="23.976";
    #    $ofps23_data[1]=$spin_convert_ofps;
    #    $ofps29_data[0]="29.0";
    #    $ofps29_data[1]=$spin_convert_ofps;

    $nfps25_data[0]="25.0";
    $nfps25_data[1]=$spin_convert_nfps;
    $nfps23_data[0]="23.976";
    $nfps23_data[1]=$spin_convert_nfps;
    $nfps29_data[0]="29.0";
    $nfps29_data[1]=$spin_convert_nfps;

    #signals to Convert option menu
    #    $ofps25_item->signal_connect( 'activate', \&GSubEditor::callbacks::FPS, \@ofps25_data );
    #    $ofps23_item->signal_connect( 'activate', \&GSubEditor::callbacks::FPS, \@ofps23_data );
    #    $ofps29_item->signal_connect( 'activate', \&GSubEditor::callbacks::FPS, \@ofps29_data );

    $nfps25_item->signal_connect( 'activate', \&GSubEditor::callbacks::FPS, \@nfps25_data );
    $nfps23_item->signal_connect( 'activate', \&GSubEditor::callbacks::FPS, \@nfps23_data );
    $nfps29_item->signal_connect( 'activate', \&GSubEditor::callbacks::FPS, \@nfps29_data );

    #Shifting callbackdata
    $s_data[0] = $adj_shift_time;
    $s_data[1] = $adj_shift_start;
    $s_data[2] = $adj_shift_end;
    $s_data[3] = $menuitem_file_reread;
    $s_data[4] = $menuitem_file_save;
    $s_data[5] = $model_all;
    $s_data[6] = $button_shift_shift;
    $s_data[7] = $button_convert_convert;
    $s_data[8] = $menuitem_file_save;

    #toStart callbackdata
    $ts_data[0] = $list_all;
    $ts_data[1] = $adj_shift_start;

    #toEnd callbackdata
    $te_data[0] = $list_all;
    $te_data[1] = $adj_shift_end;

    #convertFPS callbackdata
    $cfps_data[0] = $label_convert_cfps; #was $spin_convert_ofps;
    $cfps_data[1] = $spin_convert_nfps;
    $cfps_data[2] = $menuitem_file_reread;
    $cfps_data[3] = $menuitem_file_save;
    $cfps_data[4] =  $model_all;
    $cfps_data[5] =  $button_shift_shift;
    $cfps_data[6] =  $button_convert_convert;
    $cfps_data[7] =  $checkbox_convert_time;

    #applyChangesToFile callback data
    #    $apply_data[0] =  $model_all;
    #    $apply_data[1] =  $button_shift_shift;
    #    $apply_data[2] =  $button_convert_convert;
    #    $apply_data[3] =  $menuitem_file_saveas;
    #    $apply_data[4] =  $menuitem_file_reread;

    $ss_data[0] = $list_all;	#list_all
    $ss_data[1] = $adj_split_second;

    $split_data[0] = $adj_split_second; #second starts
    $split_data[1] = $adj_split_secondstarts; #second starts

    $add_data[0] = $list_all;

    $add_data[1] = $button_all_remove;
    $add_data[2]=$table_shift;
    $add_data[3]=$table_convert;
    $add_data[4]=$button_all_remove;
    $add_data[5]=$button_all_preview;
    $add_data[6]=$menuitem_file_saveas;
    $add_data[7]=$table_split;
    $add_data[8]=$menuitem_edit_join;

    $table_split->set_sensitive(0);
    $table_shift->set_sensitive(0);
    $table_convert->set_sensitive(0);
    $button_all_remove->set_sensitive(0);
    $button_all_preview->set_sensitive(0);
    $menuitem_file_save->set_sensitive(0);

    # signals window
    $button_quit->signal_connect( "clicked", \&GSubEditor::callbacks::quitProgram, $window );

    $button_shift_start->signal_connect( "clicked", \&GSubEditor::callbacks::toStart, \@ts_data );
    $button_shift_end->signal_connect( "clicked", \&GSubEditor::callbacks::toEnd, \@te_data);
    $button_shift_shift->signal_connect( "clicked", \&GSubEditor::callbacks::shifting, \@s_data );

    $button_convert_convert->signal_connect( "clicked", \&GSubEditor::callbacks::convertFPS, \@cfps_data);

    #    $button_split_second->signal_connect( "clicked", \&GSubEditor::callbacks::toStart, \@ss_data );
    $button_split_save->signal_connect( "clicked", \&GSubEditor::callbacks::splitSave, \@split_data);

    $button_all_add->signal_connect( "clicked", \&GSubEditor::callbacks::addSubtitle, \@add_data );
    $button_all_remove->signal_connect( "clicked", \&GSubEditor::callbacks::removeSubtitle, \@add_data );
    $button_all_preview->signal_connect( "clicked", \&GSubEditor::callbacks::preview, $list_all );

    $window->signal_connect( "delete_event" => \&GSubEditor::callbacks::delete_event );
    $window->signal_connect(destroy => sub { Gtk2->main_quit; });

    $window->set_border_width( 1 );
    $window->add( $table_main );

    $window->show_all();
    #    $button_split_second->hide();
    #    $button_all_apply->hide();
    #    $button_all_apply->hide();
  }

1;
__END__

=head1 NAME

GSubEditor - Perl extension only for gsubeditor script

=head1 SYNOPSIS

  use GSubEditor;

=head1 DESCRIPTION

  Is used by gsubeditor to manage subtitles with Gtk2-interface.

=head2 EXPORT

None by default.

=head1 SEE ALSO

Subtitle(pm3), GSubEditor(pm3), GSubEditor::callbacks(pm3), GSubEditor::locale(pm3), GSubEditor::onfig(pm3)

http://www.kapsi.fi/~jvaltane/subtitle/
http://www.kapsi.fi/~jvaltane/gsubeditor/

=head1 AUTHOR

Joni Valtanen, E<lt>jvaltane@kapsi.fiE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006 by Joni Valtanen

This library is free software; you can redistribute it and/or modify
it under the GNU General Public Licence version 2.

Read COPYING from the package for further information.


=cut
